declare class ReactiveWebStorage {
    constructor(prefix: any, webStorage: any, reactiveStorageAdapter: any);
    /**
     * Key used in webStorage
     *
     * @param {string} key The key used by reactiveWebStorage
     * @returns {string} The key used by webStorage
     */
    obtainWebStorageKey(key: string): string;
    /**
     * Key used in reactiveWebStorage
     *
     * @param {string} key The key used by reactiveWebStorage
     * @returns {string} The key used by reactiveStorageAdapter
     */
    obtainReactiveStorageAdapterKey(key: string): string;
    get reactiveStorageAdapter(): any;
    get reactiveStorage(): any;
    get prefix(): any;
    /**
     * Obtains the number of elements saved in reactiveWebStorage.
     *
     * @returns {number} Number of elements saved in reactiveWebStorage.
     * @readonly
     */
    readonly get length(): number;
    /**
     * Returns the key in nth position into reactiveWebStorage.
     *
     * @param {number} index The index of a key in the reactiveWebStorage.
     * @returns {string} The key in nth position.
     */
    key(index: number): string;
    /**
     * Returns the parsed key's value saved into reactiveWebStorage.
     *
     * @param {string} key A key saved into reactiveWebStorage.
     * @returns {string | null} The key's value.
     */
    getItem(key: string): string | null;
    /**
     * Saves the pair key/value into reactiveWebStorage.
     *
     * @param {string} key A key saved into reactiveWebStorage.
     * @param {string} item The key's value to save.
     */
    setItem(key: string, item: string): void;
    /**
     * Removes the pair key/value from reactiveWebStorage.
     *
     * @param {string} key The key to remove from reactiveWebStorage.
     */
    removeItem(key: string): void;
    /** Removes all pairs key/value into reactiveWebStorage. */
    clear(): void;
    /**
     * This method must be used into listener object that listens an event. Sets
     * the data from localStorage into reactiveLocalStorage when the listened
     * event is fired.
     */
    loadDataFromWebStorage(): void;
    #private;
}

declare module '@vue/runtime-core' {
    /**
     * When the user installs this plugin and he calls
     * app.config.globalProperties, it will appear this information.
     */
    interface ComponentCustomProperties {
        $reactiveWebStorage: ReactiveWebStorage;
    }
}

declare class ReactiveWebStorageError extends Error {
}

declare class ReactiveStorageAdapter {
    constructor(reactiveStorage: any);
    get reactiveStorage(): any;
    /**
     * Obtains the number of elements saved in reactiveStorage.
     *
     * @returns {number} Number of elements saved in reactiveStorage.
     * @readonly
     */
    readonly get length(): number;
    /**
     * Returns the key in nth position into reactiveStorage.
     *
     * @param {number} index The index of a key in the reactiveStorage.
     * @returns {string} The key in nth position.
     */
    key(index: number): string;
    /**
     * Saves the pair key/value into reactiveStorage.
     *
     * @param {string} key A key saved into reactiveStorage.
     * @param {string} item The key's value to save.
     */
    setItem(key: string, item: string): void;
    /**
     * Returns the parsed key's value saved into reactiveStorage.
     *
     * @param {string} key A key saved into reactiveStorage.
     * @returns {string | null} The key's value.
     */
    getItem(key: string): string | null;
    /**
     * Removes the pair key/value from reactiveStorage.
     *
     * @param {string} key The key to remove from reactiveStorage.
     */
    removeItem(key: string): void;
    /** Removes all pairs key/value into reactiveStorage. */
    clear(): void;
    #private;
}

/**
 * Creates an installer used to install this plugin.
 *
 * @returns {ReactiveWebStorageInstaller} The installer object that will install
 *   this plugin.
 */
declare function createReactiveWebStorageInstaller(): ReactiveWebStorageInstaller;

export { ReactiveStorageAdapter, ReactiveWebStorage, ReactiveWebStorageError, createReactiveWebStorageInstaller as default };
